/*
*   sliplogin 2.2
*   92/10/15 John DuBois (john@armory.com)
*   93/11/14 Added 38400 rate.
*   93/11/16 Modified to read /etc/hosts.slip instead of getting info from
*            hostname() & /etc/passwd, since this doesn't work with routing.
*            Added -x option.
*   94/03/11 Added tty checking.
*   94/06/12 Added option for extra arguments; made '-' equiv. to no tty field.
*   95/12/30 Added n option.  Removed +e as an always-supplied option.
*            Ported to v5 compiler: use consistent declaration style.
*            Ported to v5 slattach: give netmask if on OS > v4
*/
#ifdef _M_SYSV
#define M_SYSV
#endif
#include <memory.h>
#include <stdarg.h>
#include <sys/termio.h>
#ifdef M_SYSV
#include <sys/types.h>
#include <sys/socket.h>
#else
#include <sys/param.h>
#endif
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <string.h>
#ifdef M_XENIX
#include <sys/utsname.h>
#endif

char **StrToArr(char *hsLine,char sep[],char *esc);
char *defread(char *pattern);
int CheckTTY(char *TTYs,char *tty);

char *Comment = "@(#) sliplogin 1.1 96/07/03";

WriteStrings(int fd, ...)
{
    va_list pvar;
    char *S;

    va_start(pvar,fd);
    while (S = va_arg(pvar,char *))
	write(fd,S,strlen(S));
    va_end(pvar);
}

main(int argc, char **argv)
{
    char *tty;
    char *remotehost;
    char *localhost;
    char *netmask;
    char *hsLine;
    char **Fields;

    struct termio TTYParm;
    char *Rates[] = { "0","50","75","110","134","150","200","300",
		    "600","1200","1800","2400","4800","9600","19200","38400" };
    char *login;
    int debug = 0;
    int noslattach = 0;
    char **argptr = argv + 1;
    char *ttyList;
    int NumArg;
    char **Args, **sArgPtr;
#ifdef M_XENIX
    struct scoutsname info;
    char *release;
    int old_slattach;
#endif

    if (argc > 1 && !strcmp(*argptr,"-x")) {
	debug = 1;
	argptr++;
	argc--;
    }
    if (argc > 1 && !strcmp(*argptr,"-n")) {
	debug = 1;
	noslattach = 1;
	argptr++;
	argc--;
    }
    if (!(tty = ttyname(0))) {
	WriteStrings(2,argv[0],": ",strerror(errno),"\n",(char *) 0);
	exit(1);
    }
    if (ioctl(0,TCGETA,&TTYParm)) {
	WriteStrings(2,argv[0],": ",strerror(errno),"\n",(char *) 0);
	exit(1);
    }
    if (argc > 1) {
	if (!getuid()) {
	    login = *argptr;
	}
	else {
	    WriteStrings(2,argv[0],": you are not the superuser.\n",(char *) 0);
	    exit(1);
	}
    }
    else
	if (!(login = getlogin())) {
	    WriteStrings(2,argv[0],": could not get login name.\n",(char *) 0);
	    exit(1);
	}
    if (defopen("/etc/hosts.slip")) {
	WriteStrings(2,argv[0],": could not open /etc/hosts.slip\n",
	(char *) 0);
	exit(1);
    }
    if (!(hsLine = defread(login)) || *hsLine != ' ' && *hsLine != '\t') {
	WriteStrings(2,argv[0],
	": could not find ",login," in /etc/hosts.slip\n",(char *) 0);
	exit(1);
    }
    /* hsLine now points to the space/tab after the login name */
    Fields = StrToArr(hsLine," \t",'\0');
    if (!Fields || !Fields[1] || !Fields[2]) {
	WriteStrings(2,argv[0],
	": not enough fields in /etc/hosts.slip record for ",login,"\n",
	(char *) 0);
	exit(1);
    }
    netmask = *Fields;
    localhost = *++Fields;
    remotehost = *++Fields;
    /* If a tty list field was given, check if this user can log in on it. */
    if (ttyList = *++Fields) {
	if (strcmp(ttyList,"-") && CheckTTY(ttyList,tty)) {
	    WriteStrings(2,argv[0],": may not be run by ",login," on ",tty,
	    ".\n", (char *) 0);
	    exit(1);
	}
	++Fields;
    }
#ifdef M_XENIX
    __scoinfo(&info, sizeof(info));
    release = info.release;
    old_slattach = release[4] < '5';
    if (debug)
	WriteStrings(2,"Release=",release," slattach-version=",old_slattach ?
	"old" : "new","\n",(char *)0);
#endif
    /* Find out how many extra-args there are */
    for (argptr = Fields; *argptr; argptr++)
	;
    NumArg = argptr - Fields;
    /* allocate space for the arg vector to be passed to slattach */
    sArgPtr = Args = (char **) calloc(NumArg + 7,sizeof(char *));
    *(sArgPtr++) = "slattach";
/*    *(sArgPtr++) = "+e";	*/
    for (; *Fields; Fields++)
	*(sArgPtr++) = *Fields;
    *(sArgPtr++) = tty;
    *(sArgPtr++) = localhost;
    *(sArgPtr++) = remotehost;
#ifdef M_XENIX
    if (!old_slattach)
	*(sArgPtr++) = netmask;
#endif
    *(sArgPtr++) = Rates[TTYParm.c_cflag & CBAUD];
    *sArgPtr = (char *) 0;
    if (debug) {
	WriteStrings(2,"tty=",tty,
	"; localhost=",localhost,"; remotehost=",remotehost,"; netmask=",
	netmask,"\n",(char *) 0);
	for (argptr = Args; *argptr; argptr++)
	    WriteStrings(2,"<",*argptr,">\n",(char *) 0);
    }
    if (!noslattach)
	execv("/etc/slattach",Args);
}

/*
    TTYs is a comma-separated list of tty names that this user may log in on.
    Each tty name may or may not include a leading /dev/.
    tty is the full path of the tty the user is logged in on.
    Return 0 if logged in on a permitted tty; 1 if not.
*/

int CheckTTY(char *TTYs, char *tty)
{
    char **TTYlist;
    char *OKtty;

    TTYlist = StrToArr(TTYs,",",'\0');
    for (; OKtty = *TTYlist; TTYlist++)
	if (!strcmp(OKtty,tty) || !strncmp(tty,"/dev/",5) &&
	!strcmp(tty+5,OKtty))
	    return 0;
    return 1;
}
